/* ------------------------------------------------------------
 * This header file defines the constants for the PIC16F877A
 * For Serial LCD
 * Author: Ober Choo
 * Modified: 14 Dec 2007
 * PICC Lite copiler is required
 * For Initialize SPI for 16F877A, User need to to use different 
 * command base on what version of PICC Lite they use. The command 
 * used in this program is for PICC Lite Version 9.60.
 * ----------------------------------------------------------*/
#include <pic.h>

__CONFIG ( 0x3F32 ) ;
#define SLCD_E	 		RD0	// E of SLCD is connected to RD0
#define DUMMY_DATA		0xff


// declaration of function prototype

// initialize the IO of PIC16F877A
extern void init_IO(void);

// delay for 10 us for each value 
void delay_10us (unsigned int count);

// delay for 1 ms for each value
extern void delay_ms (unsigned int count);

// Initialize SPI
extern void init_spi(void);

// Send and receive data via SPI
extern unsigned char read_write_spi(unsigned char data);

// Provide 8 cycles of clock to SPI
extern void clock_spi(void);

// Read data from SPI.
extern unsigned char read_spi(void);

// Write data to SPI
extern void write_spi(unsigned char data);

// Output the data to the correct pin
extern void send_slcd_data(unsigned char data, unsigned char rs_bit);

// Return cursor to home position
extern void slcd_home(void);

// Clear serial LCD screen
extern void slcd_clear(void);

// Display data in ASCII format
extern void slcd_putchar(char data);

// Jump to next line
extern void slcd_newline(void);

// Jump to address (Line one:0x00-0x0f, Line 2:0x40-0x4f)
extern void slcd_jump(unsigned char address);

// Initialize the LCD display
extern void slcd_init (void);

// Display an array of string
extern void slcd_putstr(const char* msg);

// initialize the IO of PIC16F877A
void init_IO(void)
{
	TRISD = 0b11111110;
	TRISC = 0b11010111;
	TRISB = 0b11111111;	
}

// Initialize SPI for 16F877A
void init_spi(void) 
{
	SSPOV = 0;			// Clear the overflow bit
	SMP = 0;			// Input data sampled at middle of data output time,
						// user need to type STAT_SMP instead of SMP if using previous version of PICC Lite
	CKE = 1;			// data transmitted at falling edge of clock
						// user need to type STAT_CKE instead of CKE if using previous version of PICC Lite
	CKP = 0;			// Idle state is low
	SSPM3 = 0;
	SSPM2 = 0;
	SSPM1 = 0;
	SSPM0 = 0;			// SPI master mode, clock = FOSC/4
	SSPEN = 1;			// Enable the SPI module
}


// Send and receive data via SPI
unsigned char send_receive_spi(unsigned char data) 
{
	while(BF);			// user need to type STAT_BF instead of BF if using previous version of PICC Lite
	SSPBUF = data;
	
	while(!BF);			// user need to type STAT_BF instead of BF if using previous version of PICC Lite
	data = SSPBUF;
	SSPOV = 0;
	return data;
}



// Provide 8 cycles of clock to SPI.
void clock_spi(void)
{
	send_receive_spi(DUMMY_DATA);
}



// Read data from SPI.
unsigned char read_spi(void)
{
	return send_receive_spi(DUMMY_DATA);
}



// Write data to SPI.
void write_spi(unsigned char data)
{
	send_receive_spi(data);
}


// Bitswap.
unsigned char bitswap(unsigned char data)
{
	unsigned char result=0;
	unsigned char i;
	
	for(i=0;i<8;i++)
	{
		result = result << 1;
		result |= (data & 0x01);
		data = data >> 1;
	}
	return result;
}

// Send the data to the serial LCD.
void send_slcd_data(unsigned char data, unsigned char rs_bit)
{
	write_spi(bitswap((rs_bit & 0x01) << 1));	// Higher byte.
	write_spi(bitswap(data));					// Lower byte.
	delay_10us(2);
}

// Return cursor to home position
void slcd_home(void)
{
	send_slcd_data(0x02, 0);
	SLCD_E = 0;
	delay_ms(2);
	SLCD_E = 1;
}



// Clear serial LCD screen
void slcd_clear(void)
{
	send_slcd_data(0x01,0);
	SLCD_E = 0;
	delay_ms(2);
	SLCD_E = 1;
}



// Display data in ASCII format
void slcd_putchar(char data)
{
	send_slcd_data(data, 1);
	SLCD_E = 0;
	delay_10us(10);
	SLCD_E = 1;
}



// Jump to next line
void slcd_newline(void)
{
	send_slcd_data(0xc0, 0);
	SLCD_E = 0;
	delay_10us(5);
	SLCD_E = 1;
}

// Jump to address (Line one:0x00-0x0f, Line 2:0x40-0x4f)
void slcd_jump(unsigned char address) {
	address = address + 0x80;

	send_slcd_data(address, 0);
	SLCD_E = 0;
	delay_10us(5);
	SLCD_E = 1;
}


// Initialize the LCD display
void slcd_init(void)
{
	SLCD_E = 1;
	delay_ms(15);
	
	send_slcd_data(0b00111000, 0);	// DL=1, N=1, F=0
	SLCD_E = 0;
	delay_10us(5);
	SLCD_E = 1;
	
	send_slcd_data(0b00001100, 0);	// D=1, C=0, B=0
	SLCD_E = 0;
	delay_10us(5);
	SLCD_E = 1;
	
	slcd_clear();
}


// Display an array of string
void slcd_putstr(const char * msg)
{
	while (*msg)
	{
		slcd_putchar(*msg++);
		delay_10us(1);	
	}
}


/* Delay subroutine*/
void delay_10us (unsigned int count)
{	unsigned int i;
	unsigned int a;
	for (a = 0; a <=count; a++)
	{
		for (i = 0; i<= 20;i++)
		{
		}
	}	
}


void delay_ms (unsigned int count)
{
	unsigned int a;
	for (a = 0; a <=count; a++)
	{
		delay_10us(100);
	}
}

